FFmpeg
Loading...
Searching...
No Matches
rational.c
Go to the documentation of this file.
1/*
2 * rational numbers
3 * Copyright (c) 2003 Michael Niedermayer <michaelni@gmx.at>
4 *
5 * This file is part of FFmpeg.
6 *
7 * FFmpeg is free software; you can redistribute it and/or
8 * modify it under the terms of the GNU Lesser General Public
9 * License as published by the Free Software Foundation; either
10 * version 2.1 of the License, or (at your option) any later version.
11 *
12 * FFmpeg is distributed in the hope that it will be useful,
13 * but WITHOUT ANY WARRANTY; without even the implied warranty of
14 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
15 * Lesser General Public License for more details.
16 *
17 * You should have received a copy of the GNU Lesser General Public
18 * License along with FFmpeg; if not, write to the Free Software
19 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
20 */
21
22/**
23 * @file
24 * rational numbers
25 * @author Michael Niedermayer <michaelni@gmx.at>
26 */
27
28#include "avassert.h"
29#include <limits.h>
30
31#include "common.h"
32#include "mathematics.h"
33#include "rational.h"
34
35int av_reduce(int *dst_num, int *dst_den,
36 int64_t num, int64_t den, int64_t max)
37{
38 AVRational a0 = { 0, 1 }, a1 = { 1, 0 };
39 int sign = (num < 0) ^ (den < 0);
40 int64_t gcd = av_gcd(FFABS(num), FFABS(den));
41
42 if (gcd) {
43 num = FFABS(num) / gcd;
44 den = FFABS(den) / gcd;
45 }
46 if (num <= max && den <= max) {
47 a1 = (AVRational) { num, den };
48 den = 0;
49 }
50
51 while (den) {
52 uint64_t x = num / den;
53 int64_t next_den = num - den * x;
54 int64_t a2n = x * a1.num + a0.num;
55 int64_t a2d = x * a1.den + a0.den;
56
57 if (a2n > max || a2d > max) {
58 if (a1.num) x = (max - a0.num) / a1.num;
59 if (a1.den) x = FFMIN(x, (max - a0.den) / a1.den);
60
61 if (den * (2 * x * a1.den + a0.den) > num * a1.den)
62 a1 = (AVRational) { x * a1.num + a0.num, x * a1.den + a0.den };
63 break;
64 }
65
66 a0 = a1;
67 a1 = (AVRational) { a2n, a2d };
68 num = den;
69 den = next_den;
70 }
71 av_assert2(av_gcd(a1.num, a1.den) <= 1U);
72 av_assert2(a1.num <= max && a1.den <= max);
73
74 *dst_num = sign ? -a1.num : a1.num;
75 *dst_den = a1.den;
76
77 return den == 0;
78}
79
81{
82 av_reduce(&b.num, &b.den,
83 b.num * (int64_t) c.num,
84 b.den * (int64_t) c.den, INT_MAX);
85 return b;
86}
87
89{
90 return av_mul_q(b, (AVRational) { c.den, c.num });
91}
92
94 av_reduce(&b.num, &b.den,
95 b.num * (int64_t) c.den +
96 c.num * (int64_t) b.den,
97 b.den * (int64_t) c.den, INT_MAX);
98 return b;
99}
100
102{
103 av_reduce(&b.num, &b.den,
104 b.num * (int64_t) c.den -
105 c.num * (int64_t) b.den,
106 b.den * (int64_t) c.den, INT_MAX);
107 return b;
108}
109
110AVRational av_d2q(double d, int max)
111{
113 int exponent;
114 int64_t den;
115 if (isnan(d))
116 return (AVRational) { 0,0 };
117 if (fabs(d) > INT_MAX + 3LL)
118 return (AVRational) { d < 0 ? -1 : 1, 0 };
119 frexp(d, &exponent);
120 exponent = FFMAX(exponent-1, 0);
121 den = 1LL << (62 - exponent);
122 // (int64_t)rint() and llrint() do not work with gcc on ia64 and sparc64,
123 // see Ticket2713 for affected gcc/glibc versions
124 av_reduce(&a.num, &a.den, floor(d * den + 0.5), den, max);
125
126 return a;
127}
128
130{
131 /* n/d is q, a/b is the median between q1 and q2 */
132 int64_t a = q1.num * (int64_t)q2.den + q2.num * (int64_t)q1.den;
133 int64_t b = 2 * (int64_t)q1.den * q2.den;
134
135 /* rnd_up(a*d/b) > n => a*d/b > n */
137
138 /* rnd_down(a*d/b) < n => a*d/b < n */
139 int64_t x_down = av_rescale_rnd(a, q.den, b, AV_ROUND_DOWN);
140
141 return ((x_up > q.num) - (x_down < q.num)) * av_cmp_q(q2, q1);
142}
143
145{
146 int i, nearest_q_idx = 0;
147 for (i = 0; q_list[i].den; i++)
148 if (av_nearer_q(q, q_list[i], q_list[nearest_q_idx]) > 0)
149 nearest_q_idx = i;
150
151 return nearest_q_idx;
152}
153
155 int64_t n;
156 int shift;
157 int sign = 0;
158
159 if (q.den < 0) {
160 q.den *= -1;
161 q.num *= -1;
162 }
163 if (q.num < 0) {
164 q.num *= -1;
165 sign = 1;
166 }
167
168 if (!q.num && !q.den) return 0xFFC00000;
169 if (!q.num) return 0;
170 if (!q.den) return 0x7F800000 | (q.num & 0x80000000);
171
172 shift = 23 + av_log2(q.den) - av_log2(q.num);
173 if (shift >= 0) n = av_rescale(q.num, 1LL<<shift, q.den);
174 else n = av_rescale(q.num, 1, ((int64_t)q.den) << -shift);
175
176 shift -= n >= (1<<24);
177 shift += n < (1<<23);
178
179 if (shift >= 0) n = av_rescale(q.num, 1LL<<shift, q.den);
180 else n = av_rescale(q.num, 1, ((int64_t)q.den) << -shift);
181
182 av_assert1(n < (1<<24));
183 av_assert1(n >= (1<<23));
184
185 return sign<<31 | (150-shift)<<23 | (n - (1<<23));
186}
187
189{
190 int64_t gcd, lcm;
191
192 gcd = av_gcd(a.den, b.den);
193 lcm = (a.den / gcd) * b.den;
194 return lcm < max_den ? av_make_q(av_gcd(a.num, b.num), lcm) : def;
195}
simple assert() macros that are a bit more flexible than ISO C assert().
#define av_assert2(cond)
assert() equivalent, that does lie in speed critical code.
Definition avassert.h:68
#define av_assert1(cond)
assert() equivalent, that does not lie in speed critical code.
Definition avassert.h:58
#define i(width, name, range_min, range_max)
Definition cbs_h264.c:63
common internal and external API header
#define FFABS(a)
Absolute value, Note, INT_MIN / INT64_MIN result in undefined behavior as they are not representable ...
Definition common.h:74
long long int64_t
Definition coverity.c:34
static __device__ float fabs(float a)
static __device__ float floor(float a)
#define max(a, b)
int av_nearer_q(AVRational q, AVRational q1, AVRational q2)
Find which of the two rationals is closer to another rational.
Definition rational.c:129
AVRational av_add_q(AVRational b, AVRational c)
Add two rationals.
Definition rational.c:93
AVRational av_mul_q(AVRational b, AVRational c)
Multiply two rationals.
Definition rational.c:80
int av_reduce(int *dst_num, int *dst_den, int64_t num, int64_t den, int64_t max)
Reduce a fraction.
Definition rational.c:35
int av_find_nearest_q_idx(AVRational q, const AVRational *q_list)
Find the value in a list of rationals nearest a given reference rational.
Definition rational.c:144
AVRational av_gcd_q(AVRational a, AVRational b, int max_den, AVRational def)
Return the best rational so that a and b are multiple of it.
Definition rational.c:188
static AVRational av_make_q(int num, int den)
Create an AVRational.
Definition rational.h:71
AVRational av_d2q(double d, int max)
Convert a double precision floating point number to a rational.
Definition rational.c:110
static int av_cmp_q(AVRational a, AVRational b)
Compare two rationals.
Definition rational.h:89
AVRational av_sub_q(AVRational b, AVRational c)
Subtract one rational from another.
Definition rational.c:101
uint32_t av_q2intfloat(AVRational q)
Convert an AVRational to a IEEE 32-bit float expressed in fixed-point format.
Definition rational.c:154
AVRational av_div_q(AVRational b, AVRational c)
Divide one rational by another.
Definition rational.c:88
int64_t av_rescale(int64_t a, int64_t b, int64_t c)
Rescale a 64-bit integer with rounding to nearest.
int64_t av_rescale_rnd(int64_t a, int64_t b, int64_t c, enum AVRounding rnd)
Rescale a 64-bit integer with specified rounding.
Definition mathematics.c:58
int64_t av_gcd(int64_t a, int64_t b)
Compute the greatest common divisor of two integer operands.
Definition mathematics.c:37
@ AV_ROUND_DOWN
Round toward -infinity.
@ AV_ROUND_UP
Round toward +infinity.
int a
#define b
Definition input.c:43
#define av_log2
Definition intmath.h:84
static int shift(int a, int b)
Definition bonk.c:261
#define isnan(x)
Definition libm.h:342
#define FFMIN(a, b)
Definition macros.h:49
#define FFMAX(a, b)
Definition macros.h:47
Utilities for rational number calculation.
Rational number (pair of numerator and denominator).
Definition rational.h:58
int num
Numerator.
Definition rational.h:59
int den
Denominator.
Definition rational.h:60
static const uint8_t q1[256]
Definition twofish.c:100
static double a0(void *priv, double x, double y)
Definition vf_xfade.c:2028
static double a1(void *priv, double x, double y)
Definition vf_xfade.c:2029
static double c[64]